/*
 * Copyright (C) 2014 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef ServicesController_h
#define ServicesController_h

#if ENABLE(SERVICE_CONTROLS)

#include <wtf/Forward.h>
#include <wtf/Noncopyable.h>
#include <wtf/RetainPtr.h>

namespace WebKit {

class ServicesController {
    WTF_MAKE_NONCOPYABLE(ServicesController);
    friend NeverDestroyed<ServicesController>;
public:
    static ServicesController& singleton();

    bool hasImageServices() const { return m_hasImageServices; }
    bool hasSelectionServices() const { return m_hasSelectionServices; }
    bool hasRichContentServices() const { return m_hasRichContentServices; }

    void refreshExistingServices(bool refreshImmediately = true);

private:
    ServicesController();

    dispatch_queue_t m_refreshQueue;
    std::atomic_bool m_hasPendingRefresh;

    std::atomic<bool> m_hasImageServices;
    std::atomic<bool> m_hasSelectionServices;
    std::atomic<bool> m_hasRichContentServices;

    bool m_lastSentHasImageServices;
    bool m_lastSentHasSelectionServices;
    bool m_lastSentHasRichContentServices;

    RetainPtr<id> m_extensionWatcher;
    RetainPtr<id> m_uiExtensionWatcher;
};

} // namespace WebKit

#endif // ENABLE(SERVICE_CONTROLS)
#endif // ServicesController_h
